;
; Player bank sits at $8000
; Is swapped in when not playing the current NSF.
;

; gamepad values
PAD_A      := $01
PAD_B      := $02
PAD_SELECT := $04
PAD_START  := $08
PAD_U      := $10
PAD_D      := $20
PAD_L      := $40
PAD_R      := $80

OAM        := $0400

; from ramcode.s
.importzp gamepad
.importzp temp_word0
.import bank_add
.import last_8000
.import pal
.import fps
.import frame
.import sec0
.import sec1
.import min0
.import min1
.import play_addr
.import init_addr
.import track_nsf
.import pal_nsf
.import pal_adjust
.import pal_adjust_on
.import main_exit
.import play
.import player_on
.import play_through
.import play_seconds
.import track
.import menu_hold
.import gfx_pal
.import gfx_nmt
.import gfx_latch
.import gfx_len
.import gfx_ready

.import sta_5FF8, sta_5FF9, sta_5FFA, sta_5FFB, sta_5FFC, sta_5FFD, sta_5FFE, sta_5FFF
.import main_loop


.segment "TRAMP_NMI"
    jmp nmi
.segment "TRAMP_MAIN"
    jmp main_return
.segment "TRAMP_TRACK"
    jmp track_play

.segment "CODE"

nmi:
    lda gfx_ready
    beq @gfx_end
    ; copy palette
    lda #$3F
    sta $2006
    lda #$00
    sta $2006
    ldx #0
    :
        lda gfx_pal, x
        sta $2007
        inx
        cpx #32
    bne :-
    ; copy nametable
    lda gfx_latch+0
    sta $2006
    lda gfx_latch+1
    sta $2006
    ldx #0
    cpx gfx_len
    beq :++ ; no nametable to update
    :
        lda gfx_nmt, x
        sta $2007
        inx
        cpx gfx_len
    bne :-
    :
    ; OAM DMA
    lda #0
    sta $2003
    lda #>OAM
    sta $4014
    ; reset scroll
    lda #0
    sta $2005
    sta $2005
    ; enable rendering
    lda #%00011110
    sta $2001
    ; mark gfx_ready false
    lda #0
    sta gfx_ready
@gfx_end:
    lda player_on
    beq :+
        jsr tick_player
    :
    rts

main_return:
    lda #0
    sta play
    sta player_on
    sta $4015
    ; remove continuous play or pause indicator
    lda #$FF
    sta OAM+4
    sta OAM+8
    ; if main_exit == 2 continue to next track instead
    lda main_exit
    cmp #2
    bne :+
        jmp track_play
    :
    ; return to startup.s
    lda #$FF
    sta $5FFF ; restore startup bank
    rts

;
; track playback
;

pal_intro: .byte $0F, $15, $25, $27, $0F, $15, $05, $27
pal_earth: .byte $0F, $19, $29, $27, $0F, $19, $09, $27
pal_air:   .byte $0F, $28, $38, $27, $0F, $28, $18, $27
pal_fire:  .byte $0F, $16, $26, $27, $0F, $16, $06, $27
pal_water: .byte $0F, $11, $21, $27, $0F, $11, $01, $27
pal_void:  .byte $0F, $13, $23, $27, $0F, $13, $03, $27
track_pal:
    .word pal_intro ; 00
    .word pal_earth ; 01
    .word pal_earth ; 02
    .word pal_earth ; 03
    .word pal_air   ; 04
    .word pal_air   ; 05
    .word pal_air   ; 06
    .word pal_water ; 07
    .word pal_water ; 08
    .word pal_water ; 09
    .word pal_intro ; 10
    .word pal_fire  ; 12
    .word pal_fire  ; 13
    .word pal_fire  ; 14
    .word pal_void  ; 15
    .word pal_void  ; 16
    .word pal_void  ; 17

title_00: .asciiz "INTRODUCTION                "
title_01: .asciiz "DRY UP AND TUMBLE ON        "
title_02: .asciiz "GARTEN                      "
title_03: .asciiz "MILKY FIELDS                "
title_04: .asciiz "POLLUTION                   "
title_05: .asciiz "SKY BUS FOR HIRE            "
title_06: .asciiz "EXTENDS LEVANT              "
title_07: .asciiz "TIDE/GROTTO                 "
title_08: .asciiz "BE WATER MY FRIEND          "
title_09: .asciiz "THE BLOOP                   "
title_10: .asciiz "DREAMGATE                   "
title_11: .asciiz "MAGMO'S MAGICAL INGOT       "
title_12: .asciiz "OUROBOROS                   "
title_13: .asciiz "AN EARTH MADE OF MOLTEN ROCK"
title_14: .asciiz "THE ROBOT EMPTINESS         "
title_15: .asciiz "DOVE DOME                   "
title_16: .asciiz "NULL AND VOID               "
track_title:
    .word title_00
    .word title_01
    .word title_02
    .word title_03
    .word title_04
    .word title_05
    .word title_06
    .word title_07
    .word title_08
    .word title_09
    .word title_10
    .word title_11
    .word title_12
    .word title_13
    .word title_14
    .word title_15
    .word title_16
TITLE_POS := $2000+2+(6*32)

artist_00: .asciiz "JRLEPAGE         "
artist_01: .asciiz "BRADSMITH        "
artist_02: .asciiz "IPI              "
artist_03: .asciiz "TQ-JAM           "
artist_04: .asciiz "DOXIC            "
artist_05: .asciiz "COMMANDYCAN      "
artist_06: .asciiz "HERTZDEVIL       "
artist_07: .asciiz "JANGLER          "
artist_08: .asciiz "TRISTENDO        "
artist_09: .asciiz "KFARADAY         "
artist_10: .asciiz "STINKBUG         "
artist_11: .asciiz "GYMS             "
artist_12: .asciiz "MOOT BOOXL%      "
artist_13: .asciiz "FEAROFDARK       "
artist_14: .asciiz "CHIPCHOP         "
artist_15: .asciiz "ZAN-ZAN-ZAWA-VEIA"
artist_16: .asciiz "KUNGFUFURBY      "
artist_name:
    .word artist_00
    .word artist_01
    .word artist_02
    .word artist_03
    .word artist_04
    .word artist_05
    .word artist_06
    .word artist_07
    .word artist_08
    .word artist_09
    .word artist_10
    .word artist_11
    .word artist_12
    .word artist_13
    .word artist_14
    .word artist_15
    .word artist_16
ARTIST_POS := $2000+2+(8*32)

number_00: .asciiz "0 "
number_01: .asciiz "1 "
number_02: .asciiz "2 "
number_03: .asciiz "3 "
number_04: .asciiz "4 "
number_05: .asciiz "5 "
number_06: .asciiz "6 "
number_07: .asciiz "7 "
number_08: .asciiz "8 "
number_09: .asciiz "9 "
number_10: .asciiz "10"
number_11: .asciiz "11"
number_12: .asciiz "12"
number_13: .asciiz "13"
number_14: .asciiz "14"
number_15: .asciiz "15"
number_16: .asciiz "16"
track_number:
    .word number_00
    .word number_01
    .word number_02
    .word number_03
    .word number_04
    .word number_05
    .word number_06
    .word number_07
    .word number_08
    .word number_09
    .word number_10
    .word number_11
    .word number_12
    .word number_13
    .word number_14
    .word number_15
    .word number_16
NUMBER_POS := $2000+10+(19*32)

render_string:
    ldy #0
    :
        lda (temp_word0), y
        beq :+
        clc
        adc #$20 ; my ASCII set is off by $20, go figure
        sta $2007
        iny
    jmp :-
    :
    rts

track_draw_info:
    lda track
    asl
    tax
    ; track name
    lda track_title+0, x
    sta temp_word0+0
    lda track_title+1, x
    sta temp_word0+1
    lda #>TITLE_POS
    sta $2006
    lda #<TITLE_POS
    sta $2006
    jsr render_string
    ; artist name
    lda artist_name+0, x
    sta temp_word0+0
    lda artist_name+1, x
    sta temp_word0+1
    lda #>ARTIST_POS
    sta $2006
    lda #<ARTIST_POS
    sta $2006
    jsr render_string
    ; track number
    lda track_number+0, x
    sta temp_word0+0
    lda track_number+1, x
    sta temp_word0+1
    lda #>NUMBER_POS
    sta $2006
    lda #<NUMBER_POS
    sta $2006
    jsr render_string
    ; track palette
    lda track_pal+0, x
    sta temp_word0+0
    lda track_pal+1, x
    sta temp_word0+1
    ldy #0
    :
        lda (temp_word0), y
        sta gfx_pal+8, y
        iny
        cpy #8
    bne :-
    rts

track_init_addr: ; INIT from NSF header
.word $8008 ; 00
.word $8008 ; 01
.word $8008 ; 02
.word $8080 ; 03
.word $8000 ; 04
.word $8008 ; 05
.word $8000 ; 06
.word $AA38 ; 07
.word $8008 ; 08
.word $8008 ; 09
.word $8008 ; 10
.word $8008 ; 11
.word $AA38 ; 12
.word $8008 ; 13
.word $AA1F ; 14
.word $8000 ; 15
.word $8000 ; 16

track_play_addr: ; PLAY from NSF header
.word $800B ; 00
.word $800B ; 01
.word $800B ; 02
.word $8084 ; 03
.word $8003 ; 04
.word $800B ; 05
.word $8003 ; 06
.word $AA3B ; 07
.word $800B ; 08
.word $800B ; 09
.word $800B ; 10
.word $800B ; 11
.word $AA3B ; 12
.word $800B ; 13
.word $AA22 ; 14
.word $8003 ; 15
.word $8003 ; 16

track_bank_offset:
;      00   01   02   03   04   05   06   07
.byte $01, $09, $11, $24, $30, $38, $47, $58
;      08   09   10   11   12   13   14   15   16
.byte $60, $7C, $8A, $92, $A1, $A9, $B7, $BF, $C9

track_bank_start: ; startup banks (from NSF header)
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 00
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 01
.byte $00, $01, $02, $03, $04, $05, $06, $12 ; 02
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 03
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 04
.byte $00, $01, $02, $03, $0B, $0C, $0D, $0E ; 05
.byte $00, $01, $02, $03, $0D, $0E, $0F, $10 ; 06
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 07
.byte $00, $01, $02, $03, $04, $05, $06, $1B ; 08
.byte $00, $01, $02, $03, $0A, $0B, $0C, $0D ; 09
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 10
.byte $00, $01, $02, $03, $0B, $0C, $0D, $0E ; 11
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 12
.byte $00, $01, $02, $03, $0A, $0B, $0C, $0D ; 13
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 14
.byte $00, $01, $02, $03, $06, $07, $08, $09 ; 15
.byte $00, $01, $02, $03, $04, $05, $06, $07 ; 16

track_pal_adjust: ; double every 5th frame if in PAL
.byte 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1

track_time: ; time to play each track in seconds
    .word ( 3*60)+33 ; 00
    .word ( 2*60)+22 ; 01
    .word ( 2*60)+10 ; 02
    .word ( 3*60)+26 ; 03
    .word ( 3*60)+44 ; 04
    .word ( 3*60)+10 ; 05
    .word ( 8*60)+28 ; 06
    .word ( 4*60)+11 ; 07
    .word ( 3*60)+41 ; 08
    .word ( 3*60)+29 ; 09
    .word ( 2*60)+07 ; 10
    .word ( 4*60)+12 ; 11
    .word ( 2*60)+32 ; 12
    .word ( 4*60)+25 ; 13
    .word ( 2*60)+55 ; 14
    .word ( 4*60)+52 ; 15
    .word (11*60)+10 ; 16

track_play:
.ifndef NSF
    jsr track_draw_info
.endif
    ; reset clock and other variables
    lda #0
    sta menu_hold
    sta main_exit
    sta frame
    sta sec0
    sta sec1
    sta min0
    sta min1
    sta play_seconds+0
    sta play_seconds+1
    ; PAL/DENDY adjustment
    lda pal
    cmp #2 ; DENDY
    bne :+
        lda #1
        sta pal_adjust_on
        lda #0
        sta pal_nsf ; play as if NTSC
        jmp @pal_adjust_end
    :
    cmp #1 ; PAL
    bne :+
        ldx track
        lda track_pal_adjust, x
        sta pal_adjust_on
        eor #1
        sta pal_nsf ; play as if PAL unless adjust is on
        jmp @pal_adjust_end
    :
    lda #0 ; NTSC
    sta pal_adjust_on
    sta pal_nsf
@pal_adjust_end:
    lda #0
    sta pal_adjust
    sta track_nsf
    ; setup init/play
    lda track
    asl
    tax
    lda track_init_addr+0, x
    sta init_addr+0
    lda track_init_addr+1, x
    sta init_addr+1
    lda track_play_addr+0, x
    sta play_addr+0
    lda track_play_addr+1, x
    sta play_addr+1
    ; setup banks
    lda track
    tax
    lda track_bank_offset, x
    sta bank_add
    lda track
    asl
    asl
    asl
    tax
    lda track_bank_start+0, x
    clc
    adc bank_add
    cmp #$24 ; replace tuning table for 03_00
    bne :++
        pha
        lda pal
        cmp #1
        bne :+
            pla
            lda #$FC
            jmp :++
        :
        pla
    :
    cmp #$C9 ; replace tuning table for 10_00
    bne :++
        pha
        lda pal
        cmp #1
        bne :+
            pla
            lda #$FB
            jmp :++
        :
        pla
    :
    sta last_8000 ; instead of banking this page right now, store this for when we jmp main_loop
    lda track_bank_start+1, x
    jsr sta_5FF9
    lda track_bank_start+2, x
    jsr sta_5FFA
    lda track_bank_start+3, x
    jsr sta_5FFB
    lda track_bank_start+4, x
    jsr sta_5FFC
    lda track_bank_start+5, x
    jsr sta_5FFD
    lda track_bank_start+6, x
    jsr sta_5FFE
    lda track_bank_start+7, x
    jsr sta_5FFF
    ; clear ZP for NSF
    lda #0
    ldx #0
    :
        sta $00, x
        inx
        cpx #$40
    bne :-
    ; clear RAM for NSF
    ldx #0
    :
        sta $200, x
        sta $300, x
        inx
    bne :-
    ; clear $4011
    lda #0
    sta $4011
    ; turn on the player
    lda #1
    sta play
    sta player_on
.ifdef NSF
    rts
.else
    ; jump to RAM loop
    jmp main_loop
.endif

CLOCK_POS := $2000+5+(16*32)

tick_player:
    jsr gamepad_poll
    ; if input is clear, we are ready to take a new button press
    lda gamepad
    bne :+
        lda #1
        sta menu_hold
        jmp :++
    :
    lda menu_hold
    beq @done_input
    lda gamepad
    beq :+
        lda #0
        sta menu_hold       
    :
    ; select quits to menu
    lda gamepad
    and #(PAD_SELECT)
    beq :+
        lda #1
        sta main_exit
    :
    ; start pauses audio
    lda gamepad
    and #(PAD_START)
    beq :++
        lda play
        beq :+ ; pause and mute
            lda #0
            sta play
            sta $4015
            jmp :++
        : ; else unpause and unmute
        lda #1
        sta play
        lda #%00001111
        sta $4015
    :
    ; B = single track
    lda gamepad
    and #(PAD_B)
    beq :+
        lda #0
        sta play_through
    :
    ; A = continuous
    lda gamepad
    and #(PAD_A)
    beq :+
        lda #1
        sta play_through
    :
    ; left/up = decrement track
    lda gamepad
    and #(PAD_L | PAD_U)
    beq :+
        lda track
        beq :+
        dec track
        lda #2
        sta main_exit
    :
    ; right = increment track
    lda gamepad
    and #(PAD_R | PAD_D)
    beq :+
        lda track
        cmp #16
        beq :+
        inc track
        lda #2
        sta main_exit
    :
@done_input:
    ; advance clock
    lda play
    beq @clock_end
    inc frame
    lda frame
    cmp fps
    beq :+
        bcc @clock_end ; if ! (frame > fps) don't tick seconds
    :
    lda #0
    sta frame
    ; increment internal seconds counter
    inc play_seconds+0
    bne :+
        inc play_seconds+1
    :
    ; increment clock
    inc sec0
    lda sec0
    cmp #10
    bne @clock_end
    lda #0
    sta sec0
    inc sec1
    lda sec1
    cmp #6
    bne @clock_end
    lda #0
    sta sec1
    inc min0
    lda min0
    cmp #10
    bne @clock_end
    lda #0
    sta min0
    inc min1
@clock_end:
    ; check track timer for song stop/advance
    lda track
    asl
    tax
    lda track_time+1, x
    cmp play_seconds+1
    beq :+
        bcs @time_check_end ; track_time high > play_seconds high
        jmp @time_over ; track_time_high < play_seconds high
    : ; track_time_high == play_seconds_high
    lda track_time+0, x
    cmp play_seconds+0
    beq @time_over
    bcs @time_check_end
@time_over:
    lda #0
    sta play ; stop track
    sta $4015 ; mute track
    lda track ; don't skip if track 16
    cmp #16
    beq :+
        lda play_through
        beq :+ ; if continuous play, advance track
        inc track
        lda #2
        sta main_exit
    :
@time_check_end:
    ; draw clock
    lda min1
    clc
    adc #$50
    sta gfx_nmt+0
    lda min0
    clc
    adc #$50
    sta gfx_nmt+1
    lda #$5A
    sta gfx_nmt+2
    lda sec1
    clc
    adc #$50
    sta gfx_nmt+3
    lda sec0
    clc
    adc #$50
    sta gfx_nmt+4
    lda #5
    sta gfx_len
    lda #>CLOCK_POS
    sta gfx_latch+0
    lda #<CLOCK_POS
    sta gfx_latch+1
    lda #1
    ; draw continuous play cursor and/or pause
    lda play
    bne :+
        lda #$FF
        sta OAM+4 ; hide continuous play
        lda #165
        sta OAM+8 ; show pause
        jmp @sprites_end
    :
    lda #$FF
    sta OAM+8 ; hide pause
    ;
    lda play_through
    beq :+
        lda #165
        sta OAM+4
        jmp :++
    :
        lda #$FF
        sta OAM+4
    :
@sprites_end:
    ; done draw
    sta gfx_ready
    rts

;
; INPUT
;

gamepad_poll:
    ; latch the current controller state
    lda #1
    sta $4016
    lda #0
    sta $4016
    ; store high bit in gamepad to mark end of read
    lda #%10000000
    sta gamepad
    ; read 8 bits from controller port
    :
        lda $4016
        and #%00000011
        cmp #%00000001
        ror gamepad
    bcc :-
    ; DPCM conflict may have corrupted first read, test again to make sure
@gamepad_reread:
    lda gamepad
    pha ; store previous read on the stack
    lda #1
    sta $4016
    lda #0
    sta $4016
    lda #%10000000
    sta gamepad
    :
        lda $4016
        and #%00000011
        cmp #%00000001
        ror gamepad
    bcc :-
    pla ; pop the first read to compare
    cmp gamepad
    bne @gamepad_reread
    rts

; end of file