;
; startup bank at F000
;
; 

; gamepad values
PAD_A      := $01
PAD_B      := $02
PAD_SELECT := $04
PAD_START  := $08
PAD_U      := $10
PAD_D      := $20
PAD_L      := $40
PAD_R      := $80

OAM        := $0400

; from ramcode.s
.importzp nmi_count
.importzp gamepad
.importzp temp_word0
.importzp temp_word1
.import pal
.import fps
.import track
.import play_through
.import menu_pos
.import menu_hold
.import menu_blip
.import gfx_pal
.import gfx_nmt
.import gfx_latch
.import gfx_len
.import gfx_ready

; from startup.cfg
.import __RAMCODE_LOAD__
.import __RAMCODE_RUN__
.import __RAMCODE_SIZE__

; from detect_pal.s
.import getTVSystem

.segment "CODE"

;
; TITLE
;

do_title:
    lda #0
    sta menu_pos
    sta menu_hold
    sta gfx_len
    jsr setup_screen_title
    jsr render_on
@loop:
    jsr wait_nmi
    jsr tick_blip
    jsr gamepad_poll
    ; if input is clear, we are ready to take a new button press
    lda gamepad
    bne :+
        lda #1
        sta menu_hold
        jmp :++
    :
    lda menu_hold
    beq @done_input
    lda gamepad
    beq :+
        lda #0
        sta menu_hold       
    :
    ; respond to input
    lda gamepad
    and #(PAD_U | PAD_D | PAD_SELECT)
    beq :+
        inc menu_pos
        lda menu_pos
        and #1
        sta menu_pos
        jsr make_blip
    :
    lda gamepad
    and #(PAD_A | PAD_B | PAD_START)
    beq :++
        jsr render_off
        lda menu_pos
        bne :+
            jmp do_menu
        :
        jmp do_info
    :
@done_input:
    ; move sprite
    lda #96
    sta OAM+3 ; sprite 0 x position
    lda menu_pos
    bne :+
        lda #151 ; START
        jmp :++
    :
        lda #167 ; INFO
    :
    sta OAM+0 ; sprite 0 y position
    ; graphics are ready to update
    lda #1
    sta gfx_ready
    jmp @loop

;
; INFO
;

build_string: .asciiz "NES"
ntsc_string:  .asciiz "NTSC "
pal_string:   .asciiz "PAL  "
dendy_string: .asciiz "DENDY"
tv_strings:
    .word ntsc_string
    .word pal_string
    .word dendy_string
TV_STRING_POS    := $2000+ 1+(27*32)
BUILD_STRING_POS := $2000+27+(27*32)

do_info:
    lda #0
    sta menu_hold
    sta gfx_len
    ; move the cursor offscreen
    lda #$FF
    sta OAM+0
    ; draw the screen
    jsr setup_screen_info
    ldx #8
    :
        lda pal_fire-8, x
        sta gfx_pal, x
        sta gfx_pal+16, x
        inx
        cpx #16
    bne :-
    ; draw system detected
    lda #>TV_STRING_POS
    sta $2006
    lda #<TV_STRING_POS
    sta $2006
    lda pal
    asl
    tax
    lda tv_strings+0, x
    sta temp_word0+0
    lda tv_strings+1, x
    sta temp_word0+1
    jsr render_string
    ; draw build number
    lda #>BUILD_STRING_POS
    sta $2006
    lda #<BUILD_STRING_POS
    sta $2006
    lda #<build_string
    sta temp_word0+0
    lda #>build_string
    sta temp_word0+1
    jsr render_string
    ; ready
    jsr render_on
@loop:
    jsr wait_nmi
    jsr tick_blip
    jsr gamepad_poll
    ; if input is clear, we are ready to take a new button press
    lda gamepad
    bne :+
        lda #1
        sta menu_hold
        jmp :++
    :
    lda menu_hold
    beq @done_input
    lda gamepad
    beq :+
        lda #0
        sta menu_hold       
    :
    ; just reset back to title on a button press
    lda gamepad
    and #(PAD_A | PAD_B | PAD_START)
    beq :+
        jsr render_off
        jmp power_on
    :
@done_input:
    jmp @loop

;
; MENU
;

ARTIST_NAME_LEN := 17
ARTIST_STRING_POS := $2000+11+(26*32)
artist_00: .asciiz "JRLEPAGE         "
artist_01: .asciiz "BRADSMITH        "
artist_02: .asciiz "IPI              "
artist_03: .asciiz "TQ-JAM           "
artist_04: .asciiz "DOXIC            "
artist_05: .asciiz "COMMANDYCAN      "
artist_06: .asciiz "HERTZDEVIL       "
artist_07: .asciiz "JANGLER          "
artist_08: .asciiz "TRISTENDO        "
artist_09: .asciiz "KFARADAY         "
artist_10: .asciiz "STINKBUG         "
artist_11: .asciiz "GYMS             "
artist_12: .asciiz "MOOT BOOXL%      "
artist_13: .asciiz "FEAROFDARK       "
artist_14: .asciiz "CHIPCHOP         "
artist_15: .asciiz "ZAN-ZAN-ZAWA-VEIA"
artist_16: .asciiz "KUNGFUFURBY      "
artist_strings:
    .word artist_00
    .word artist_01
    .word artist_02
    .word artist_03
    .word artist_04
    .word artist_05
    .word artist_06
    .word artist_07
    .word artist_08
    .word artist_09
    .word artist_10
    .word artist_11
    .word artist_12
    .word artist_13
    .word artist_14
    .word artist_15
    .word artist_16

pal_intro: .byte $0F, $15, $25, $27, $0F, $15, $05, $27
pal_earth: .byte $0F, $19, $29, $27, $0F, $19, $09, $27
pal_air:   .byte $0F, $28, $38, $27, $0F, $28, $18, $27
pal_fire:  .byte $0F, $06, $16, $27, $0F, $06, $07, $27
pal_water: .byte $0F, $11, $21, $27, $0F, $11, $01, $27
pal_void:  .byte $0F, $13, $23, $27, $0F, $13, $03, $27
track_pal:
    .word pal_intro ; 00
    .word pal_earth ; 01
    .word pal_earth ; 02
    .word pal_earth ; 03
    .word pal_air   ; 04
    .word pal_air   ; 05
    .word pal_air   ; 06
    .word pal_water ; 07
    .word pal_water ; 08
    .word pal_water ; 09
    .word pal_intro ; 10
    .word pal_fire  ; 12
    .word pal_fire  ; 13
    .word pal_fire  ; 14
    .word pal_void  ; 15
    .word pal_void  ; 16
    .word pal_void  ; 17

menu_render_artist:
    lda track
    asl
    tax
    lda artist_strings+0, x
    sta temp_word0+0
    lda artist_strings+1, x
    sta temp_word0+1
    ldy #0
    :
        lda (temp_word0), y
        clc
        adc #$20 ; my ascii is off by $20 sorry
        sta gfx_nmt, y
        iny
        cpy #ARTIST_NAME_LEN
    bne :-
    lda #16
    sta OAM+3 ; sprite X position
    lda track
    asl
    asl
    asl
    clc
    adc #((7*8)-1)
    sta OAM+0 ; sprite Y position
    ; palette for song
    lda track
    asl
    tax
    lda track_pal+0, x
    sta temp_word0+0
    lda track_pal+1, x
    sta temp_word0+1
    ldy #0
    :
        lda (temp_word0), y
        sta gfx_pal+8, y
        iny
        cpy #8
    bne :-
    rts

do_menu:
    lda #0
    sta menu_hold
    sta $4011 ; reset DMC counter
    jsr setup_screen_menu
    jsr menu_render_artist
    lda #>ARTIST_STRING_POS
    sta gfx_latch+0
    lda #<ARTIST_STRING_POS
    sta gfx_latch+1
    lda #ARTIST_NAME_LEN
    sta gfx_len
    jsr render_on
@loop:
    jsr wait_nmi
    jsr tick_blip
    jsr gamepad_poll
    ; if input is clear, we are ready to take a new button press
    lda gamepad
    bne :+
        lda #1
        sta menu_hold
        jmp :++
    :
    lda menu_hold
    beq @done_input
    lda gamepad
    beq :+
        lda #0
        sta menu_hold       
    :
    ; respond to input
    lda gamepad
    and #(PAD_D | PAD_SELECT)
    beq :++
        inc track
        lda track
        cmp #17
        bne :+
           lda #0
           sta track
        :
        jsr make_blip
    :
    lda gamepad
    and #(PAD_U)
    beq :++
        dec track
        lda track
        cmp #$FF
        bne :+
           lda #16
           sta track
        :
        jsr make_blip
    :
    lda gamepad
    and #(PAD_SELECT)
    beq :+
        jsr render_off
        jmp power_on
    :
    lda gamepad
    and #(PAD_B)
    beq :+
        jsr render_off
        lda #0
        sta play_through
        jmp do_track
    :
    lda gamepad
    and #(PAD_A | PAD_START)
    beq :+
        jsr render_off
        lda #1
        sta play_through
        jmp do_track
    :
@done_input:
    ; update artist graphics
    jsr menu_render_artist
    ; graphics are ready to update
    lda #1
    sta gfx_ready
    jmp @loop

;
; TRACK
;

do_track:
    lda #0
    sta menu_hold
    jsr setup_screen_track
    lda #$FF
    sta OAM+0 ; move sprite offscreen
    jsr $8006 ; player.s track_play
    jmp do_menu

;
; RENDER HELPERS
;

wait_nmi:
    lda nmi_count
    :
        cmp nmi_count
    beq :-
    rts    

render_off:
    jsr wait_nmi
    lda #0
    sta $2000
    sta $2001
    rts

render_on:
    lda #1
    sta gfx_ready
    lda #%10000000
    sta $2000
    rts

render_string:
    ldy #0
    :
        lda (temp_word0), y
        beq :+
        clc
        adc #$20 ; my ASCII set is off by $20, go figure
        sta $2007
        iny
    jmp :-
    :
    rts

;
; BLIP SOUND
;

BLIP_LEN := 5

blip_freq: .byte $00, $08, $10, $30, $20, $40
blip_vol:  .byte $30, $31, $32, $33, $33, $34

tick_blip:
    lda menu_blip
    bne :+ ; no blip if zero
        lda #0 ; mute
        sta $4015
        rts
    :
    cmp #BLIP_LEN
    bne :+ ; start up sound on first frame of blip
        lda #%00000001 ; enable square 1
        sta $4015
        ;
        lda blip_vol + BLIP_LEN ; volume
        sta $4000
        lda #%00000111 ; sweep (disabled)
        sta $4001
        lda blip_freq + BLIP_LEN ; low frequency
        sta $4002
        lda #%00001000 ; start channel, set high freq to 0
        sta $4003
        dec menu_blip
        jmp :++
    : ; else just tick sound
        ldx menu_blip
        lda blip_vol, x
        sta $4000
        lda blip_freq, x
        sta $4002
        dec menu_blip
    :
    rts

make_blip:
    lda #BLIP_LEN
    sta menu_blip
    rts

;
; INPUT
;

gamepad_poll:
    ; latch the current controller state
    lda #1
    sta $4016
    lda #0
    sta $4016
    ; store high bit in gamepad to mark end of read
    lda #%10000000
    sta gamepad
    ; read 8 bits from controller port
    :
        lda $4016
        and #%00000011
        cmp #%00000001
        ror gamepad
    bcc :-
    rts

;
; SETUP STUFF
;

load_chr_ram:
    lda #$FA
    sta $5FFE ; switch FA bank into E000
    lda #0
    sta $2006 ; latch PPU to 0000
    sta $2006
    jsr load_chr_ram_4k
    jsr load_chr_ram_4k
    rts
load_chr_ram_4k:
    lda #$00
    sta temp_word0+0
    lda #$E0
    sta temp_word0+1
    ldx #0
    :
        ldy #0
        :
            lda (temp_word0), y
            sta $2007
            iny
        bne :-
        inc temp_word0+1
        inx
        cpx #16
    bne :--
    rts

load_ramcode:
    ldy #0
    ; temp_word0 = source
    lda #<__RAMCODE_LOAD__
    sta temp_word0+0
    lda #>__RAMCODE_LOAD__
    sta temp_word0+1
    ; temp_word1 = destination
    lda #<__RAMCODE_RUN__
    sta temp_word1+0
    lda #>__RAMCODE_RUN__
    sta temp_word1+1
@loop:
    ; check if finished
    lda temp_word0+1
    cmp #>(__RAMCODE_LOAD__ + __RAMCODE_SIZE__)
    bne :+
    lda temp_word0+0
    cmp #<(__RAMCODE_LOAD__ + __RAMCODE_SIZE__)
    bne :+
    rts ; finished
    :
    ; copy a byte
    lda (temp_word0),Y
    sta (temp_word1),Y
    ; increment source/destination
    inc temp_word0+0
    bne :+
        inc temp_word0+1
    :
    inc temp_word1+0
    bne :+
        inc temp_word1+1
    :
    jmp @loop

title_palette:
    .byte $0F,$00,$10,$30
    .byte $0F,$01,$21,$31
    .byte $0F,$16,$26,$27
    .byte $0F,$16,$06,$27
setup_palette:
    ldx #0
    :
        lda title_palette, x
        sta gfx_pal, x
        sta gfx_pal+16, x
        inx
        cpx #16
    bne :-
    rts

setup_oam:
    lda #$FF
    ldx #0
    :
        sta OAM, x
        inx
    bne :-
    ; setup sprite 0 as our menu cursor (initially offscren)
    lda #$FF
    sta OAM+0 ; Y
    lda #$46 ; &
    sta OAM+1 ; tile
    lda #%00000001
    sta OAM+2 ; attrib
    lda #0
    sta OAM+3 ; X
    ; setup sprite 1 as our continue-play cursor
    lda #$FF
    sta OAM+4
    lda #$7E ; ^
    sta OAM+5
    lda #%00000001
    sta OAM+6
    lda #56
    sta OAM+7
    ; setup sprite 2 as our pause cursor
    lda #$FF
    sta OAM+8
    lda #$44 ; $
    sta OAM+9
    lda #%00000001
    sta OAM+10
    lda #56
    sta OAM+11
    rts

setup_screen_title:
    lda #$E0
    sta temp_word0+1
    jmp copy_screen
setup_screen_info:
    lda #$E4
    sta temp_word0+1
    jmp copy_screen
setup_screen_menu:
    lda #$E8
    sta temp_word0+1
    jmp copy_screen
setup_screen_track:
    lda #$EC
    sta temp_word0+1
    jmp copy_screen
copy_screen:
    lda #$00
    sta temp_word0+0
    lda #$FE ; screens are in FE bank
    sta $5FFE ; switch FE bank into E000
    lda #$20
    sta $2006
    lda #$00
    sta $2006
    ldy #0
    .repeat 4
        :
            lda (temp_word0),Y
            sta $2007
            iny
        bne :-
        inc temp_word0+1
    .endrepeat
    rts

;
; POWER ON
;

power_on:
    ; standard startup
    sei       ; set interrupt flag (unnecessary, unless reset is called from code)
    cld       ; disable decimal mode
    ldx #$40
    stx $4017 ; disable APU IRQ
    ldx $ff
    txs       ; set up stack
    ldx #$00
    stx $2000 ; disable NMI
    stx $2001 ; disable render
    stx $4010 ; disable DPCM IRQ
    stx $4015 ; mute APU
    ;
    bit $2002 ; clear vblank flag
    ; wait for vblank
    :
        bit $2002
    bpl :-
    ; clear memory
    lda #$00
    tax
    :
        sta $0000, X
        sta $0100, X
        sta $0200, X
        sta $0300, X
        sta $0400, X
        sta $0500, X
        sta $0600, X
        sta $0700, X
        inx
    bne :-
    ; wait for second vblank
    :
        bit $2002
    bpl :-
    ;
    ; PPU is now warmed up, NES is ready to go!
    ;
    jsr load_chr_ram
    jsr load_ramcode
    ;
    ; detect NTSC/PAL/DENDY
    ;
    lda #%10000000
    sta $2000
    jsr getTVSystem
    cmp #1 ; 1 == PAL
    bne :+
        sta pal
        lda #50
        sta fps
        jmp @system_detected
    :
    cmp #2 ; 2 == DENDY
    bne :+
        sta pal
        lda #50
        sta fps
        jmp @system_detected
    :
    ; else presume NTSC
    lda #0
    sta pal
    lda #60
    sta fps
@system_detected:
    lda #0
    sta $2000 ; disable NMI
    jsr setup_palette
    jsr setup_oam
    jmp do_title

;
; NSF
;

.ifdef NSF

.import nsf_ramcode_init
.segment "NSF_INIT"
    pha ; temporarily store track to play
    cpx #1 ; 1 == PAL
    bne :+
        lda #1
        sta pal
        lda #50
        sta fps
        jmp :++
    :
    ; else presume NTSC
    lda #0
    sta pal
    lda #60
    sta fps
:
    jsr load_ramcode
    pla
    jmp nsf_ramcode_init

.endif

;
; VECTORS
;

.segment "VECTORS"
.import vec_nmi
.import vec_irq
.word vec_nmi
.word power_on
.word vec_irq

; end of file
